precision highp float;

uniform vec2 resolution;
uniform vec4 effectColor;
uniform vec2 effectWeight;

varying vec2 v_texCoords;

uniform sampler2D u_texture;


// BLUR9 (used by stroke)
// Blur Function
vec4 blur9(sampler2D image, vec2 uv, vec2 resolution, vec2 direction) {
    vec4 color = vec4(0.0);
    vec2 off1 = vec2(1.3846153846) * direction;
    vec2 off2 = vec2(3.2307692308) * direction;
    color += texture2D(image, uv) * 0.2270270270;
    color += texture2D(image, uv + (off1 / resolution)) * 0.3162162162;
    color += texture2D(image, uv - (off1 / resolution)) * 0.3162162162;
    color += texture2D(image, uv + (off2 / resolution)) * 0.0702702703;
    color += texture2D(image, uv - (off2 / resolution)) * 0.0702702703;
    return color;
}


// LAPLACE FILTER ALPHA
float laplaceFilterAlpha(sampler2D tex, vec2 uv, vec2 filterStep, vec2 direction) {

    // Blurred Picking Algorithm
    vec2 resolution = resolution.xy;

    // Cardinal Directions: N, S, E, W
    float alpha = 1.0 * texture2D(tex, uv).a;
    alpha = mix(alpha, 1.0, blur9(tex, uv + vec2(filterStep.x, 0.0), resolution, direction).a);
    alpha = mix(alpha, 1.0, blur9(tex, uv + vec2(-filterStep.x, 0.0), resolution, direction).a);
    alpha = mix(alpha, 1.0, blur9(tex, uv + vec2(0.0, filterStep.y), resolution, direction).a);
    alpha = mix(alpha, 1.0, blur9(tex, uv + vec2(0.0, -filterStep.y), resolution, direction).a);

    // 45-degree directions: NE, NW, SE, SW
    vec2 filterAngles = 0.7071 * filterStep; // Constant for sin/cos at 45 degree angles +
    alpha = mix(alpha, 1.0, blur9(tex, uv + vec2(filterAngles.x, filterAngles.y), resolution, direction).a);
    alpha = mix(alpha, 1.0, blur9(tex, uv + vec2(-filterAngles.x, -filterAngles.y), resolution, direction).a);
    alpha = mix(alpha, 1.0, blur9(tex, uv + vec2(-filterAngles.x, filterAngles.y), resolution, direction).a);
    alpha = mix(alpha, 1.0, blur9(tex, uv + vec2(filterAngles.x, -filterAngles.y), resolution, direction).a);


    return alpha;
}



// LAPLACE
vec4 laplace(vec4 color, sampler2D tex, vec2 uv, vec2 filterStep)
{
    float minSmoothing = 0.1;
    float maxSmoothing = 0.95;

    vec2 direction = vec2(effectWeight.x * 0.1, effectWeight.y * 0.1);

    float alphaLapAvg = (laplaceFilterAlpha(tex, uv, filterStep, vec2(0, direction.y)) + laplaceFilterAlpha(tex, uv, filterStep, vec2(direction.x, 0))) / 2.0;
    float alpha = smoothstep(minSmoothing, maxSmoothing, alphaLapAvg);

    vec4 result = vec4(1.0, 1.0, 1.0, alpha) * color;
    return result;
}


// MAIN SHADER
void main() {
    vec2 filterStep = effectWeight.xy / resolution.xy;
    vec2 uv = v_texCoords.xy;
    gl_FragColor = laplace(effectColor, u_texture, uv, filterStep);
}